@@ -20,10 +20,20 @@
 (declare-function ollama-buddy--model-supports-vision "ollama-buddy")
 (declare-function ollama-buddy--check-context-before-send "ollama-buddy")
 (declare-function ollama-buddy--find-reasoning-marker "ollama-buddy")
+(declare-function ollama-buddy--insert-thinking-header "ollama-buddy")
+(declare-function ollama-buddy--collapse-thinking-block "ollama-buddy")
 (declare-function ollama-buddy--update-token-rate-display "ollama-buddy")
 (declare-function ollama-buddy--send-next-in-sequence "ollama-buddy")
+(declare-function ollama-buddy--multishot-cancel-timer "ollama-buddy")
 (declare-function ollama-buddy--autosave-transcript "ollama-buddy")
 
+;; Buffer-local variables defined in ollama-buddy.el and used here as free vars.
+;; Declared to suppress byte-compile warnings; their true definitions live in
+;; ollama-buddy.el where they are defvar-local on the chat buffer.
+(defvar ollama-buddy--thinking-api-active)
+(defvar ollama-buddy--thinking-arrow-marker)
+(defvar ollama-buddy--thinking-block-start)
+
 ;; Curl-specific variables
 (defvar ollama-buddy-curl--headers-processed nil
   "Flag to track if HTTP headers have been processed for current curl request.")
@@ -217,24 +227,70 @@ When complete, CALLBACK is called with the status response and result."
         (let* ((json-data (json-read-from-string json-line))
                (message-data (alist-get 'message json-data))
                (content (when message-data (alist-get 'content message-data)))
+               (thinking-text (when message-data (alist-get 'thinking message-data)))
                (done (alist-get 'done json-data)))
-          
+
+          ;; Handle thinking-API tokens (e.g. deepseek-r1)
+          (when (and thinking-text (not (string-empty-p thinking-text)))
+            (ollama-buddy-curl--handle-thinking thinking-text))
+
           (when content
             (ollama-buddy-curl--handle-content content))
-          
+
           (when (eq done t)
             (ollama-buddy-curl--handle-completion json-data)))
       (error
        (message "Error parsing JSON: %s" (error-message-string err))))))
 
+(defun ollama-buddy-curl--handle-thinking (thinking-text)
+  "Handle a thinking token from the message.thinking API field."
+  (unless ollama-buddy--current-token-start-time
+    (ollama-buddy--cancel-response-wait-timer)
+    (setq ollama-buddy--current-token-start-time (float-time)
+          ollama-buddy--current-token-count 0
+          ollama-buddy--current-response "")
+    (when ollama-buddy--token-update-timer
+      (cancel-timer ollama-buddy--token-update-timer))
+    (setq ollama-buddy--token-update-timer
+          (run-with-timer 0 ollama-buddy--token-update-interval
+                          #'ollama-buddy--update-token-rate-display)))
+  (setq ollama-buddy--current-token-count (1+ ollama-buddy--current-token-count))
+  (with-current-buffer ollama-buddy--chat-buffer
+    (let* ((inhibit-read-only t)
+           (window (get-buffer-window ollama-buddy--chat-buffer t))
+           (old-point (and window (window-point window)))
+           (was-at-end (and window (>= old-point (point-max))))
+           (old-window-start (and window (window-start window))))
+      (save-excursion
+        (goto-char (point-max))
+        (cond
+         (ollama-buddy-collapse-thinking
+          (unless ollama-buddy--thinking-api-active
+            (setq ollama-buddy--thinking-api-active t
+                  ollama-buddy--thinking-arrow-marker (ollama-buddy--insert-thinking-header)
+                  ollama-buddy--thinking-block-start  (copy-marker (point) nil)))
+          (insert thinking-text))
+         (ollama-buddy-hide-reasoning
+          (setq ollama-buddy--thinking-api-active t))
+         (t
+          (insert thinking-text))))
+      (when window
+        (cond
+         ((and was-at-end ollama-buddy-auto-scroll)
+          (set-window-point window (point-max)))
+         (t
+          (set-window-point window old-point)
+          (set-window-start window old-window-start t)))))))
+
 (defun ollama-buddy-curl--handle-content (content)
   "Handle content token from curl response."
   ;; Initialize token tracking on first content
   (unless ollama-buddy--current-token-start-time
+    (ollama-buddy--cancel-response-wait-timer)
     (setq ollama-buddy--current-token-start-time (float-time)
           ollama-buddy--current-token-count 0
           ollama-buddy--current-response "")
-    
+
     ;; Start token rate timer
     (when ollama-buddy--token-update-timer
       (cancel-timer ollama-buddy--token-update-timer))
@@ -262,12 +318,48 @@ When complete, CALLBACK is called with the status response and result."
         (save-excursion
           (goto-char (point-max))
 
-          ;; Handle reasoning hiding
+          ;; Close thinking-API block on first content token (deepseek-r1 style)
+          (when (and ollama-buddy--thinking-api-active
+                     (not (string-empty-p content)))
+            (setq ollama-buddy--thinking-api-active nil)
+            (when (and ollama-buddy-collapse-thinking
+                       ollama-buddy--thinking-block-start)
+              (ollama-buddy--collapse-thinking-block
+               (point-max)
+               ollama-buddy--thinking-arrow-marker)
+              (set-marker ollama-buddy--thinking-block-start nil)
+              (setq ollama-buddy--thinking-block-start  nil
+                    ollama-buddy--thinking-arrow-marker nil)))
+
+          ;; Handle thinking block / reasoning hiding
           (let ((should-show-content t))
-            (when ollama-buddy-hide-reasoning
-              (let ((reasoning-marker (ollama-buddy--find-reasoning-marker content)))
+            (let ((reasoning-marker (ollama-buddy--find-reasoning-marker content)))
+              (cond
+               ;; --- Collapse mode ---
+               (ollama-buddy-collapse-thinking
+                (cond
+                 ((and reasoning-marker (eq (car reasoning-marker) 'start))
+                  (setq ollama-buddy--in-reasoning-section t
+                        should-show-content nil
+                        ollama-buddy--thinking-arrow-marker (ollama-buddy--insert-thinking-header)
+                        ollama-buddy--thinking-block-start  (copy-marker (point) nil)))
+                 ((and reasoning-marker (eq (car reasoning-marker) 'end)
+                       ollama-buddy--in-reasoning-section)
+                  (setq ollama-buddy--in-reasoning-section nil
+                        should-show-content nil)
+                  (when ollama-buddy--thinking-block-start
+                    (ollama-buddy--collapse-thinking-block
+                     (point-max)
+                     ollama-buddy--thinking-arrow-marker)
+                    (set-marker ollama-buddy--thinking-block-start nil)
+                    (setq ollama-buddy--thinking-block-start  nil
+                          ollama-buddy--thinking-arrow-marker nil)))
+                 ;; Inside block: stream normally
+                 (ollama-buddy--in-reasoning-section
+                  (setq should-show-content t))))
+               ;; --- Hide mode ---
+               (ollama-buddy-hide-reasoning
                 (cond
-                 ;; Start of reasoning section
                  ((and reasoning-marker (eq (car reasoning-marker) 'start))
                   (setq ollama-buddy--in-reasoning-section t
                         should-show-content nil)
@@ -276,18 +368,15 @@ When complete, CALLBACK is called with the status response and result."
                                    (replace-regexp-in-string
                                     "[<>]" ""
                                     (cadr reasoning-marker))))))
-                 ;; End of reasoning section
                  ((and reasoning-marker (eq (car reasoning-marker) 'end))
                   (setq ollama-buddy--in-reasoning-section nil
                         should-show-content nil)
-                  ;; Remove the reasoning indicator if present
                   (when (re-search-backward "\\[.*\\.\\.\\.]" (line-beginning-position) t)
                     (delete-region (match-beginning 0) (match-end 0))))
-                 ;; Inside reasoning section
                  (ollama-buddy--in-reasoning-section
-                  (setq should-show-content nil)))))
+                  (setq should-show-content nil))))))
 
-            ;; Insert content if not hidden
+            ;; Insert content if not suppressed
             (when should-show-content
               (insert content))))
 
@@ -318,9 +407,11 @@ When complete, CALLBACK is called with the status response and result."
                                  :tokens ollama-buddy--current-token-count
                                  :elapsed elapsed
                                  :rate rate
+                                 :wait-time ollama-buddy--response-wait-duration
                                  :timestamp (current-time))))
-          (push token-info ollama-buddy--token-usage-history))
-        
+          (push token-info ollama-buddy--token-usage-history)
+          (ollama-buddy--trim-token-history))
+
         ;; Add to history
         (ollama-buddy--add-to-history "user" ollama-buddy--current-prompt)
         (ollama-buddy--add-to-history "assistant" ollama-buddy--current-response)
@@ -372,12 +463,20 @@ When complete, CALLBACK is called with the status response and result."
               ;; Move to prompt if response is wholly visible
               (ollama-buddy--maybe-goto-prompt window response-start))))
         
+        ;; Clean up response wait timer
+        (ollama-buddy--cancel-response-wait-timer)
+
         ;; Reset tracking variables
         (setq ollama-buddy--current-token-count 0
               ollama-buddy--current-token-start-time nil
               ollama-buddy--current-response ""
               ollama-buddy--in-reasoning-section nil
               ollama-buddy-curl--headers-processed nil)
+        (when ollama-buddy--thinking-block-start
+          (set-marker ollama-buddy--thinking-block-start nil)
+          (setq ollama-buddy--thinking-block-start nil))
+        (setq ollama-buddy--thinking-api-active nil
+              ollama-buddy--thinking-arrow-marker nil)
         
         ;; Reset temporary model
         (when ollama-buddy--current-request-temporary-model
@@ -386,9 +485,11 @@ When complete, CALLBACK is called with the status response and result."
         
         ;; Handle multishot or update status
         (if ollama-buddy--multishot-sequence
-            (if (< ollama-buddy--multishot-progress (length ollama-buddy--multishot-sequence))
-                (run-with-timer 0.5 nil #'ollama-buddy--send-next-in-sequence)
-              (ollama-buddy--update-status "Multi Finished"))
+            (progn
+              (ollama-buddy--multishot-cancel-timer)
+              (if (< ollama-buddy--multishot-progress (length ollama-buddy--multishot-sequence))
+                  (run-with-timer 0.5 nil #'ollama-buddy--send-next-in-sequence)
+                (ollama-buddy--update-status "Multi Finished")))
           (let ((last-info (car ollama-buddy--token-usage-history)))
             (if last-info
                 (ollama-buddy--update-status
@@ -484,9 +585,12 @@ authentication via `ollama signin'."
                                             prompt)))))
          (messages-all (append messages-with-system (list current-message)))
          (modified-options (ollama-buddy-params-get-for-request))
-         (base-payload `((model . ,(ollama-buddy--get-real-model-name model))
-                         (messages . ,(vconcat [] messages-all))
-                         (stream . ,(if ollama-buddy-streaming-enabled t :json-false))))
+         (base-payload (append
+                        `((model . ,(ollama-buddy--get-real-model-name model))
+                          (messages . ,(vconcat [] messages-all))
+                          (stream . ,(if ollama-buddy-streaming-enabled t :json-false)))
+                        (when ollama-buddy-keepalive
+                          `((keep_alive . ,ollama-buddy-keepalive)))))
          (final-payload (if modified-options
                             (append base-payload `((options . ,modified-options)))
                           base-payload))
@@ -515,10 +619,17 @@ authentication via `ollama signin'."
         (insert (format "\n\n[Including %d attached file(s) in context]"
                         (length ollama-buddy--current-attachments))))
       
-      (if has-images
-          (insert (format "\n\n** [%s: RESPONSE with %d image(s)]"
-                          model (length image-files)))
-        (insert (format "\n\n** [%s: RESPONSE]" model)))
+      (let ((avg-wait (ollama-buddy--model-average-wait-time model)))
+        (if has-images
+            (insert (format "\n\n** [%s: RESPONSE with %d image(s)]"
+                            model (length image-files)))
+          (insert (format "\n\n** [%s: RESPONSE]" model)))
+        ;; Insert countdown estimate before the closing ]
+        (when (and avg-wait (>= avg-wait 1))
+          (backward-char 1)  ; before ]
+          (setq ollama-buddy--response-countdown-marker (copy-marker (point)))
+          (insert (format " ~%ds" (round avg-wait)))
+          (end-of-line)))
 
       (setq ollama-buddy--response-start-position (point))
       (insert "\n\n")
@@ -540,12 +651,14 @@ authentication via `ollama signin'."
         (let ((inhibit-read-only t))
           (goto-char (point-max))
           (insert "Loading response..."))))
-    
+
     (ollama-buddy--update-status (if has-images
                                      "Curl Vision Processing..."
                                    "Curl Processing...")
                                  original-model model)
 
+    (ollama-buddy--start-response-wait-timer model)
+
     ;; Kill existing process
     (when (and ollama-buddy--active-process (process-live-p ollama-buddy--active-process))
       (delete-process ollama-buddy--active-process)
