@@ -133,8 +133,13 @@ Use nil for API default behavior (adaptive)."
                                                   prompt))))))
            ;; Build the API endpoint with the model name
            (api-endpoint (format ollama-buddy-gemini-api-endpoint model-name))
-           ;; Add API key to the endpoint
+           ;; Add API key to the endpoint (non-streaming uses generateContent)
            (api-endpoint-with-key (concat api-endpoint "?key=" ollama-buddy-gemini-api-key))
+           ;; Streaming endpoint uses streamGenerateContent with alt=sse
+           (stream-api-endpoint
+            (concat (replace-regexp-in-string
+                     ":generateContent$" ":streamGenerateContent" api-endpoint)
+                    "?alt=sse&key=" ollama-buddy-gemini-api-key))
            ;; Format messages for Gemini API
            (formatted-contents (ollama-buddy-gemini--format-messages messages-all))
            ;; Build the payload for Gemini
@@ -153,64 +158,74 @@ Use nil for API default behavior (adaptive)."
            ;; Convert to JSON string
            (json-str (let ((json-encoding-pretty-print nil))
                        (ollama-buddy-escape-unicode (json-encode json-payload-with-max-tokens))))
-           (start-point (ollama-buddy-remote--prepare-chat-buffer "Google Gemini")))
+           (start-point (ollama-buddy-remote--prepare-chat-buffer "Google Gemini"))
+           (gemini-headers `(("Content-Type" . "application/json"))))
 
-      ;; Make the HTTP request
-      (let* ((url-request-method "POST")
-             (url-request-extra-headers
-              `(("Content-Type" . "application/json")))
-             (url-request-data json-str)
-             (url-mime-charset-string "utf-8")
-             (url-mime-language-string nil)
-             (url-mime-encoding-string nil)
-             (url-mime-accept-string "application/json"))
+      (if ollama-buddy-streaming-enabled
+          ;; Streaming path: use curl with SSE via streamGenerateContent
+          (ollama-buddy-remote--start-streaming-request
+           stream-api-endpoint
+           gemini-headers
+           json-str
+           #'ollama-buddy-remote--gemini-extract-content
+           "Google Gemini"
+           prompt
+           start-point)
+        ;; Non-streaming path: use url-retrieve
+        (let* ((url-request-method "POST")
+               (url-request-extra-headers gemini-headers)
+               (url-request-data json-str)
+               (url-mime-charset-string "utf-8")
+               (url-mime-language-string nil)
+               (url-mime-encoding-string nil)
+               (url-mime-accept-string "application/json"))
 
-        (url-retrieve
-         api-endpoint-with-key
-         (lambda (status)
-           (if (plist-get status :error)
-               (ollama-buddy-remote--handle-http-error
-                start-point (plist-get status :error))
-             ;; Success - process the response
-             (progn
-               (goto-char (point-min))
-             (when (re-search-forward "\n\n" nil t)
-               (let* ((json-response-raw (buffer-substring (point) (point-max)))
-                      (json-response-decoded (decode-coding-string json-response-raw 'utf-8))
-                      (json-object-type 'alist)
-                      (json-array-type 'vector)
-                      (json-key-type 'symbol))
+          (url-retrieve
+           api-endpoint-with-key
+           (lambda (status)
+             (if (plist-get status :error)
+                 (ollama-buddy-remote--handle-http-error
+                  start-point (plist-get status :error))
+               ;; Success - process the response
+               (progn
+                 (goto-char (point-min))
+                 (when (re-search-forward "\n\n" nil t)
+                   (let* ((json-response-raw (buffer-substring (point) (point-max)))
+                          (json-response-decoded (decode-coding-string json-response-raw 'utf-8))
+                          (json-object-type 'alist)
+                          (json-array-type 'vector)
+                          (json-key-type 'symbol))
 
-                 (condition-case err
-                     (let* ((json-response (json-read-from-string json-response-decoded))
-                            (error-message (alist-get 'error json-response))
-                            (content ""))
+                     (condition-case err
+                         (let* ((json-response (json-read-from-string json-response-decoded))
+                                (error-message (alist-get 'error json-response))
+                                (content ""))
 
-                       ;; Extract the message content (Gemini-specific format)
-                       (if error-message
-                           (setq content (format "Error: %s"
-                                                 (ollama-buddy-remote--format-api-error
-                                                  error-message)))
-                         ;; Parse the Gemini response structure
-                         (let* ((candidates (alist-get 'candidates json-response))
-                                (first-candidate (when (and candidates (> (length candidates) 0))
-                                                   (aref candidates 0)))
-                                (content-obj (when first-candidate
-                                               (alist-get 'content first-candidate)))
-                                (parts (when content-obj
-                                         (alist-get 'parts content-obj))))
-                           (when (and parts (> (length parts) 0))
-                             (let ((part (aref parts 0)))
-                               (setq content (alist-get 'text part))))))
+                           ;; Extract the message content (Gemini-specific format)
+                           (if error-message
+                               (setq content (format "Error: %s"
+                                                     (ollama-buddy-remote--format-api-error
+                                                      error-message)))
+                             ;; Parse the Gemini response structure
+                             (let* ((candidates (alist-get 'candidates json-response))
+                                    (first-candidate (when (and candidates (> (length candidates) 0))
+                                                       (aref candidates 0)))
+                                    (content-obj (when first-candidate
+                                                   (alist-get 'content first-candidate)))
+                                    (parts (when content-obj
+                                             (alist-get 'parts content-obj))))
+                               (when (and parts (> (length parts) 0))
+                                 (let ((part (aref parts 0)))
+                                   (setq content (alist-get 'text part))))))
 
-                       ;; Finalize the response
-                       (ollama-buddy-remote--finalize-response
-                        start-point content prompt
-                        'ollama-buddy-gemini--current-token-count))
-                   (error
-                    (ollama-buddy-remote--handle-error
-                     start-point "Gemini"
-                     (error-message-string err))))))))))))))
+                           ;; Finalize the response
+                           (ollama-buddy-remote--finalize-response
+                            start-point content prompt
+                            'ollama-buddy-gemini--current-token-count))
+                       (error
+                        (ollama-buddy-remote--handle-error
+                         start-point "Gemini"
+                         (error-message-string err)))))))))))))))
 
 (defun ollama-buddy-gemini--fetch-models ()
   "Fetch available models from Google Gemini API."
@@ -240,25 +255,32 @@ Use nil for API default behavior (adaptive)."
 
                  (condition-case err
                      (let* ((json-response (json-read-from-string json-response-raw))
-                            (models-data (alist-get 'models json-response))
-                            (models (mapcar (lambda (model-info)
-                                              (alist-get 'name model-info))
-                                            (append models-data nil)))
-                            ;; Extract just the model name part, removing the "models/" prefix
-                            (processed-models (mapcar (lambda (model)
-                                                        (if (string-match "models/\\(.*\\)" model)
-                                                            (match-string 1 model)
-                                                          model))
-                                                      models))
-                            ;; Filter to include only gemini models
-                            (chat-models (cl-remove-if-not
-                                          (lambda (model)
-                                            (string-match-p "gemini" model))
-                                          processed-models)))
+                            (models-data (append (alist-get 'models json-response) nil))
+                            ;; Build list of (short-name ctx display-name) entries
+                            (model-triples
+                             (delq nil
+                                   (mapcar (lambda (model-info)
+                                             (let* ((raw     (alist-get 'name model-info))
+                                                    (short   (if (string-match "models/\\(.*\\)" raw)
+                                                                 (match-string 1 raw)
+                                                               raw))
+                                                    (ctx     (alist-get 'inputTokenLimit model-info))
+                                                    (display (alist-get 'displayName model-info)))
+                                               (when (string-match-p "gemini" short)
+                                                 (list short ctx display))))
+                                           models-data)))
+                            (chat-models (mapcar #'car model-triples)))
                        (ollama-buddy-remote--register-models
                         ollama-buddy-gemini-marker-prefix
                         chat-models
-                        #'ollama-buddy-gemini--send))
+                        #'ollama-buddy-gemini--send)
+                       ;; Store context windows and display names in the shared metadata cache
+                       (dolist (entry model-triples)
+                         (let ((full-name (concat ollama-buddy-gemini-marker-prefix (car entry))))
+                           (puthash full-name
+                                    `((context-window . ,(cadr entry))
+                                      (display-name   . ,(caddr entry)))
+                                    ollama-buddy--models-metadata-cache))))
                    (error
                     (message "Error parsing Gemini models response: %s" (error-message-string err))
                     (ollama-buddy--update-status "Failed to parse Gemini models response"))))))))))))
