@@ -75,7 +75,7 @@ When non-nil, the LLM can invoke registered tools during conversations."
   :type 'boolean
   :group 'ollama-buddy-tools)
 
-(defcustom ollama-buddy-tools-auto-execute t
+(defcustom ollama-buddy-tools-auto-execute nil
   "Whether to automatically execute tool calls without confirmation.
 When nil, prompt the user before executing each tool call."
   :type 'boolean
@@ -87,7 +87,7 @@ This prevents infinite loops if the LLM keeps calling tools."
   :type 'integer
   :group 'ollama-buddy-tools)
 
-(defcustom ollama-buddy-tools-safe-mode t
+(defcustom ollama-buddy-tools-safe-mode nil
   "When non-nil, restrict tools to safe read-only operations.
 Disables tools that can modify files or execute arbitrary commands."
   :type 'boolean
@@ -115,9 +115,24 @@ Keys are tool names (strings), values are plists with:
 (defvar ollama-buddy-tools--pending-calls nil
   "List of pending tool calls waiting to be executed.")
 
+(defvar ollama-buddy-tools--pause-continuation nil
+  "When non-nil, suppress automatic tool-continuation after the current tool batch.
+Tools that open interactive UI sessions (e.g. ediff) set this so the LLM does
+not keep calling tools while the user is busy in that session.  The user can
+send a new message to resume the conversation normally.")
+
+(defvar ollama-buddy-tools--session-paused nil
+  "When non-nil, the chat session is paused waiting for the user to finish an
+interactive tool UI (e.g. ediff).  Shown as ⏸ in the status line.
+Cleared automatically when the user sends a new message.")
+
+(defvar ollama-buddy-tools--stop-after-batch nil
+  "When non-nil, end the tool batch without auto-continuing the LLM.
+Set when any :terminal tool executes.  Cleared after the batch.")
+
 ;;; Tool Registration
 
-(defun ollama-buddy-tools-register (name description parameters function &optional safe)
+(defun ollama-buddy-tools-register (name description parameters function &optional safe terminal)
   "Register a tool for use with Ollama Buddy.
 
 NAME is the tool name (symbol or string).
@@ -126,6 +141,9 @@ PARAMETERS is a JSON schema object describing the tool's parameters.
 FUNCTION is the Elisp function to execute when the tool is called.
   It receives an alist of arguments and should return a string result.
 SAFE indicates if the tool is safe (read-only) - defaults to nil.
+TERMINAL when non-nil, stops auto-continuation after this tool executes.
+  Use for tools that launch interactive Emacs UI sessions (e.g. ediff)
+  where you want the LLM to stop and wait for the user's next message.
 
 Example:
 
@@ -147,7 +165,8 @@ Example:
              (list :description description
                    :parameters parameters
                    :function function
-                   :safe (or safe nil))
+                   :safe (or safe nil)
+                   :terminal (or terminal nil))
              ollama-buddy-tools--registry)
     (message "Registered tool: %s" name-str)))
 
@@ -196,6 +215,22 @@ Returns a vector of tool definitions in the format expected by Ollama."
 
 ;;; Tool Execution
 
+(defun ollama-buddy-tools--format-args-for-display (arguments)
+  "Format ARGUMENTS alist for display, truncating long string values."
+  (mapconcat
+   (lambda (pair)
+     (let* ((key (car pair))
+            (val (cdr pair))
+            (val-str (if (stringp val)
+                         (if (> (length val) 60)
+                             (format "\"%s\"… (%d chars)"
+                                     (substring val 0 60) (length val))
+                           (format "%S" val))
+                       (format "%S" val))))
+       (format "%s: %s" key val-str)))
+   arguments
+   ", "))
+
 (defun ollama-buddy-tools--execute (name arguments)
   "Execute tool NAME with ARGUMENTS.
 Returns the result as a string, or an error message if execution fails."
@@ -213,11 +248,22 @@ Returns the result as a string, or an error message if execution fails."
           (error "Tool %s is not safe for execution in safe mode" name-str))
         ;; Confirmation check
         (when (and (not ollama-buddy-tools-auto-execute)
-                   (not (yes-or-no-p (format "Execute tool %s with args %S? "
-                                           name-str arguments))))
+                   (not (yes-or-no-p (format "Execute tool %s(%s)? "
+                                             name-str
+                                             (ollama-buddy-tools--format-args-for-display arguments)))))
           (error "Tool execution cancelled by user"))
-        ;; Execute the function
-        (let ((result (funcall func arguments)))
+        ;; If this tool is marked terminal, flag that auto-continuation should
+        ;; stop after this batch — set before calling in case of error.
+        (when (plist-get spec :terminal)
+          (setq ollama-buddy-tools--stop-after-batch t))
+        ;; Some models send arguments as a JSON string rather than a JSON object.
+        ;; Parse it into an alist so tool functions can use alist-get normally.
+        (let* ((parsed-args (if (stringp arguments)
+                                (condition-case nil
+                                    (json-read-from-string arguments)
+                                  (error arguments))
+                              arguments))
+               (result (funcall func parsed-args)))
           (if (stringp result)
               result
             (format "%S" result))))
@@ -334,19 +380,54 @@ Returns a list of tool result messages to append to the conversation."
     ;; list_buffers - List open buffers
     (ollama-buddy-tools-register
      'list_buffers
-     "List all open Emacs buffers"
+     "List open Emacs buffers in a structured format showing modified/read-only flags, name, size, mode and filename. Internal buffers (names starting with a space) are hidden by default unless a pattern is given."
      '((type . "object")
        (properties . ((pattern . ((type . "string")
-                                  (description . "Optional regex pattern to filter buffer names"))))))
+                                  (description . "Optional regex pattern to filter buffer names. When given, internal buffers are also included."))))))
      (lambda (args)
-       (let ((pattern (alist-get 'pattern args))
-             (buffers (mapcar #'buffer-name (buffer-list))))
-         (if pattern
-             (mapconcat #'identity
-                       (seq-filter (lambda (buf) (string-match-p pattern buf))
-                                 buffers)
-                       "\n")
-           (mapconcat #'identity buffers "\n"))))
+       (let* ((pattern (alist-get 'pattern args))
+              (buffers (seq-filter
+                        (lambda (b)
+                          (let ((name (buffer-name b)))
+                            (if pattern
+                                (string-match-p pattern name)
+                              ;; By default hide internal buffers (space-prefixed)
+                              (not (string-prefix-p " " name)))))
+                        (buffer-list)))
+              (header (format " %-2s  %-24s  %6s  %-18s %s\n %s  %s  %s  %s  %s\n"
+                              "MR" "Name" "Size" "Mode" "Filename/Process"
+                              "--" (make-string 24 ?-) "------"
+                              (make-string 18 ?-) (make-string 16 ?-)))
+              (rows (mapcar
+                     (lambda (buf)
+                       (let* ((name (buffer-name buf))
+                              (display-name (if (> (length name) 24)
+                                               (concat (substring name 0 21) "...")
+                                             name))
+                              (size (buffer-size buf))
+                              (mode-str (let ((m (format-mode-line
+                                                  (buffer-local-value 'mode-name buf)
+                                                  nil nil buf)))
+                                          (if (> (length m) 18)
+                                              (concat (substring m 0 15) "...")
+                                            m)))
+                              (modified (if (buffer-modified-p buf) "*" " "))
+                              (read-only (if (buffer-local-value 'buffer-read-only buf) "%" " "))
+                              (filename (let ((f (or (buffer-file-name buf)
+                                                     (and (local-variable-p
+                                                           'list-buffers-directory buf)
+                                                          (buffer-local-value
+                                                           'list-buffers-directory buf)))))
+                                          (if f (abbreviate-file-name f) ""))))
+                         (format " %s%s  %-24s  %6d  %-18s %s"
+                                 modified read-only display-name size mode-str filename)))
+                     buffers))
+              (total (length buffers))
+              (file-count (length (seq-filter #'buffer-file-name buffers)))
+              (footer (format "\n %d buffer%s  %d file%s"
+                              total (if (= total 1) "" "s")
+                              file-count (if (= file-count 1) "" "s"))))
+         (concat header (mapconcat #'identity rows "\n") footer)))
      t) ; safe
     
     ;; calculate - Perform calculation
@@ -392,18 +473,65 @@ Returns a list of tool result messages to append to the conversation."
            "No matches found")))
      t) ; safe
     
+    ;; propose_file_changes - Provide full new file content and review via Emacs ediff
+    (ollama-buddy-tools-register
+     'propose_file_changes
+     "Propose changes to a file by providing the COMPLETE new file content with all modifications applied. Opens Emacs ediff so the user can review every change interactively and selectively copy hunks across. Use this whenever you want to suggest edits to an existing file — provide the full updated file, not a diff or patch."
+     '((type . "object")
+       (required . ["file_path" "new_content"])
+       (properties . ((file_path . ((type . "string")
+                                    (description . "Absolute path to the existing file to modify.")))
+                      (new_content . ((type . "string")
+                                      (description . "The complete new content for the file, with all proposed changes already applied. Must be the entire file, not just a fragment or diff."))))))
+     (lambda (args)
+       (let* ((file-path (alist-get 'file_path args))
+              (new-content (alist-get 'new_content args)))
+         (cond
+          ((not (stringp file-path))
+           "Error: file_path is required (must be a string).")
+          ((not (stringp new-content))
+           "Error: new_content is required (must be a string).")
+          ((not (file-exists-p file-path))
+           (format "Error: file not found: %s" file-path))
+          (t
+           (let* ((ext (file-name-extension file-path t))
+                  (proposed-tmp (make-temp-file "ob-proposed-" nil ext)))
+             (with-temp-file proposed-tmp
+               (insert new-content))
+             (require 'ediff)
+             (let* ((proposed-buf (find-file-noselect proposed-tmp))
+                    (original-buf (find-file-noselect file-path)))
+               ;; save-window-excursion is synchronous: the window config is
+               ;; restored before Emacs redraws, so the ediff windows never
+               ;; appear on screen — but the session is registered in the
+               ;; ediff registry for the user to open with M-x eregistry.
+               (save-window-excursion
+                 (ediff-buffers proposed-buf original-buf)))
+             (format "Ediff session created silently for %s (left: proposed, right: original). Inform the user they can open it at any time with M-x ediff-show-registry — for example in a new frame or tab. In ediff: 'n'/'p' navigate differences, 'b' copies a hunk left-to-right into the original, then save the file."
+                     file-path))))))
+     nil  ; not safe — launches ediff UI
+     t)   ; terminal — stop LLM auto-continuation after ediff session created
+
     (message "Initialized %d built-in tools" (hash-table-count ollama-buddy-tools--registry))))
 
 ;;; Interactive Commands
 
 (defun ollama-buddy-tools-toggle ()
-  "Toggle tool calling on/off."
+  "Toggle tool calling on/off.
+When enabling, warns if the current model does not support tool calling."
   (interactive)
-  (setq ollama-buddy-tools-enabled (not ollama-buddy-tools-enabled))
-  ;; Refresh header line to update tool indicator
-  (when (fboundp 'ollama-buddy--update-status)
-    (ollama-buddy--update-status (or (bound-and-true-p ollama-buddy--status) "")))
-  (message "Tool calling %s" (if ollama-buddy-tools-enabled "enabled" "disabled")))
+  (let ((enabling (not ollama-buddy-tools-enabled))
+        (current-model (bound-and-true-p ollama-buddy--current-model)))
+    (if (and enabling
+             current-model
+             (fboundp 'ollama-buddy--model-supports-tools)
+             (not (ollama-buddy--model-supports-tools current-model)))
+        (message "Model '%s' does not support tool calling - tools not enabled. Switch to a tool-capable model first."
+                 current-model)
+      (setq ollama-buddy-tools-enabled enabling)
+      (when (fboundp 'ollama-buddy--update-status)
+        (ollama-buddy--update-status
+         (if ollama-buddy-tools-enabled "Tools enabled" "Tools disabled"))))))
 
 (defun ollama-buddy-tools-toggle-safe-mode ()
   "Toggle safe mode for tool execution."
@@ -417,37 +545,37 @@ Returns a list of tool result messages to append to the conversation."
   (let ((tools (ollama-buddy-tools-list))
         (buf (get-buffer-create "*Ollama Buddy Tools*")))
     (with-current-buffer buf
-      (erase-buffer)
-      (insert "#+TITLE: Ollama Buddy Tools\n\n")
-      (insert "* Status\n\n")
-      (insert (format "| Setting      | Value    |\n"))
-      (insert (format "|--------------+----------|\n"))
-      (insert (format "| Status       | %s |\n" (if ollama-buddy-tools-enabled "Enabled" "Disabled")))
-      (insert (format "| Safe Mode    | %s        |\n" (if ollama-buddy-tools-safe-mode "On" "Off")))
-      (insert (format "| Auto Execute | %s       |\n" (if ollama-buddy-tools-auto-execute "Yes" "No")))
-      (insert (format "| Total Tools  | %d        |\n\n" (length tools)))
-      (insert "* Registered Tools\n\n")
-      (dolist (name tools)
-        (let* ((spec (ollama-buddy-tools-get name))
-               (desc (plist-get spec :description))
-               (safe (plist-get spec :safe))
-               (params (plist-get spec :parameters)))
-          (insert (format "** %s\n\n" name))
-          (insert (format "- *Safety*: %s\n" (if safe "Safe (read-only)" "Unsafe (can modify)")))
-          (insert (format "- *Description*: %s\n" desc))
-          (when params
-            (insert "- *Parameters*:\n")
-            (let* ((props (alist-get 'properties params)))
-              (dolist (prop props)
-                (let ((prop-name (car prop))
-                      (prop-spec (cdr prop)))
-                  (insert (format "  - ~%s~ (%s): %s\n"
-                                  prop-name
-                                  (or (alist-get 'type prop-spec) "any")
-                                  (or (alist-get 'description prop-spec) "")))))))
-          (insert "\n")))
-      (org-mode)
-      (goto-char (point-min)))
+      (let ((inhibit-read-only t))
+        (erase-buffer)
+        (insert "#+title: Ollama Buddy Tools\n\n")
+        (insert "* Status\n\n")
+        (insert (format "- *Enabled:*      %s\n" (if ollama-buddy-tools-enabled "Yes" "No")))
+        (insert (format "- *Safe Mode:*    %s\n" (if ollama-buddy-tools-safe-mode "On" "Off")))
+        (insert (format "- *Auto Execute:* %s\n" (if ollama-buddy-tools-auto-execute "Yes" "No")))
+        (insert (format "- *Total Tools:*  %d\n\n" (length tools)))
+        (insert "* Registered Tools\n\n")
+        (dolist (name tools)
+          (let* ((spec (ollama-buddy-tools-get name))
+                 (desc (plist-get spec :description))
+                 (safe (plist-get spec :safe))
+                 (params (plist-get spec :parameters)))
+            (insert (format "** %s  [%s]\n\n" name (if safe "safe" "unsafe")))
+            (insert (format "   %s\n\n" (or desc "")))
+            (when params
+              (let ((props (alist-get 'properties params)))
+                (dolist (prop props)
+                  (let ((prop-name (car prop))
+                        (prop-spec (cdr prop)))
+                    (insert (format "   - *%s* (%s) :: %s\n"
+                                    prop-name
+                                    (or (alist-get 'type prop-spec) "any")
+                                    (or (alist-get 'description prop-spec) ""))))))
+              (insert "\n"))))
+        (org-mode)
+        (setq-local org-hide-emphasis-markers t)
+        (setq-local org-hide-leading-stars t)
+        (goto-char (point-min))
+        (view-mode 1)))
     (display-buffer buf)))
 
 ;;; Setup
