@@ -31,7 +31,6 @@
 (declare-function ollama-buddy-menu "ollama-buddy")
 (declare-function ollama-buddy-manage-models "ollama-buddy")
 (declare-function ollama-buddy--swap-model "ollama-buddy")
-(declare-function ollama-buddy--swap-model-cloud "ollama-buddy")
 (declare-function ollama-buddy-cloud-signin "ollama-buddy")
 (declare-function ollama-buddy-cloud-signout "ollama-buddy")
 (declare-function ollama-buddy-cloud-status "ollama-buddy")
@@ -69,8 +68,27 @@
 (declare-function ollama-buddy-web-search "ollama-buddy-web-search")
 (declare-function ollama-buddy-web-search-attach "ollama-buddy-web-search")
 (declare-function ollama-buddy-web-search-count "ollama-buddy-web-search")
+(declare-function ollama-buddy-set-keepalive "ollama-buddy-core")
+(declare-function ollama-buddy-toggle-airplane-mode "ollama-buddy-core")
+(declare-function ollama-buddy-toggle-in-buffer-replace "ollama-buddy-core")
 (declare-function ollama-buddy-tools-toggle "ollama-buddy-tools")
 (declare-function ollama-buddy-tools-info "ollama-buddy-tools")
+(declare-function ollama-buddy-completion-toggle "ollama-buddy-completion")
+
+(defun ollama-buddy-transient--tools-toggle ()
+  "Toggle tool calling if ollama-buddy-tools is loaded, else prompt to load it."
+  (interactive)
+  (if (fboundp 'ollama-buddy-tools-toggle)
+      (ollama-buddy-tools-toggle)
+    (message "Tool calling requires ollama-buddy-tools: (require 'ollama-buddy-tools)")))
+
+(defun ollama-buddy-transient--tools-info ()
+  "Show tool info if ollama-buddy-tools is loaded, else prompt to load it."
+  (interactive)
+  (if (fboundp 'ollama-buddy-tools-info)
+      (ollama-buddy-tools-info)
+    (message "Tool calling requires ollama-buddy-tools: (require 'ollama-buddy-tools)")))
+
 (declare-function ollama-buddy-rag-index-directory "ollama-buddy-rag")
 (declare-function ollama-buddy-rag-search "ollama-buddy-rag")
 (declare-function ollama-buddy-rag-attach "ollama-buddy-rag")
@@ -84,13 +102,13 @@
   :info-manual "(ollama-buddy)Top"
   [
    "|o(Y)o| Ollama Buddy"
-     
+   
    ["Chat"
     ("o" "Open Chat" ollama-buddy--open-chat)
-    ("A" "Attachments" ollama-buddy-transient-attachment-menu)
+    ("a" "Attachments" ollama-buddy-transient-attachment-menu)
     ("/" "Web Search" ollama-buddy-transient-web-search-menu)
     ("r" "RAG" ollama-buddy-transient-rag-menu)
-    ("a" "Authentication" ollama-buddy-transient-auth-menu)
+    ("A" "Authentication" ollama-buddy-transient-auth-menu)
     ("b" "Role Menu" ollama-buddy-role-transient-menu)
     ]
 
@@ -98,9 +116,16 @@
     ("h" "Help/Menu" ollama-buddy--menu-help-assistant)
     ("l" "Send Region" (lambda () (interactive) (ollama-buddy--send-with-command 'send-region)))
     ("k" "Cancel" ollama-buddy--cancel-request)
-    ("x" "Toggle Stream  " ollama-buddy-toggle-streaming)
-    ("W" "Toggle Tools" ollama-buddy-tools-toggle)
-    ("Q" "List Tools" ollama-buddy-tools-info)
+    ("x" "Toggle Stream" ollama-buddy-toggle-streaming)
+    ("v" "Keep Alive" ollama-buddy-set-keepalive)
+    ("!" "Airplane Mode" ollama-buddy-toggle-airplane-mode)
+    ]
+
+   ["Tools"
+    ("SPC" "Toggle Tools" ollama-buddy-transient--tools-toggle)
+    ("Q" "List Tools" ollama-buddy-transient--tools-info)
+    ("W" "In-Buffer Replace" ollama-buddy-toggle-in-buffer-replace)
+    ("c" "Completion Mode" ollama-buddy-completion-toggle)
     ]
 
    ["System Prompts"
@@ -114,9 +139,9 @@
    ["Model"
     ("M" "Manage" ollama-buddy-manage-models)
     ("m" "Switch" ollama-buddy--swap-model)
-    ("c" "Cloud" ollama-buddy--swap-model-cloud)
     ("i" "Info" ollama-buddy-show-raw-model-info)
     ("U" "Multishot" ollama-buddy--multishot-prompt)
+    ("u" "Benchmark All" ollama-buddy-benchmark-models)
     ]
 
    ["Roles"
@@ -173,12 +198,12 @@
   "Fabric patterns menu for Ollama Buddy."
   [["Fabric Prompts"
     ("s" "Send with Prompt" ollama-buddy-fabric-send)
-    ("L" "Set as System Prompt" ollama-buddy-fabric-set-system-prompt)
+    ("j" "Set as System Prompt" ollama-buddy-fabric-set-system-prompt)
     ("l" "List All Prompts" ollama-buddy-fabric-list-patterns)
     ("v" "View Prompt Details" ollama-buddy-fabric-show-pattern)
     ("S" "Sync Latest Prompts" ollama-buddy-fabric-sync-patterns)
     ("q" "Quit" ollama-buddy-transient-menu)]]
-    (interactive)
+  (interactive)
   (unless ollama-buddy-fabric--patterns
     (message "Loading Fabric patterns...")
     (ollama-buddy-fabric-populate-patterns))
@@ -277,7 +302,7 @@
   :man-page "ollama-buddy-awesome"
   [["Awesome Prompts"
     ("s" "Send with Prompt" ollama-buddy-awesome-send)
-    ("L" "Set as System Prompt" ollama-buddy-awesome-set-system-prompt)
+    ("j" "Set as System Prompt" ollama-buddy-awesome-set-system-prompt)
     ("l" "List All Prompts" ollama-buddy-awesome-list-prompts)
     ("S" "Sync Latest Prompts" ollama-buddy-awesome-sync-prompts)
     ("q" "Quit" transient-quit-one)]])
@@ -301,18 +326,18 @@
     ("x" "Sign Out" ollama-buddy-cloud-signout)
     ("s" "Status" ollama-buddy-cloud-status)]
    [:if (lambda () (featurep 'ollama-buddy-copilot))
-    "GitHub Copilot"
-    ("p" "Login" ollama-buddy-copilot-login)
-    ("l" "Logout" ollama-buddy-copilot-logout)
-    ("t" "Status" ollama-buddy-copilot-status)]
+        "GitHub Copilot"
+        ("p" "Login" ollama-buddy-copilot-login)
+        ("l" "Logout" ollama-buddy-copilot-logout)
+        ("t" "Status" ollama-buddy-copilot-status)]
    ["Navigation"
     ("q" "Quit" transient-quit-one)]])
 
 (transient-define-prefix ollama-buddy-transient-attachment-menu ()
   "File attachment menu."
   ["File Attachments"
-   ("a" "Attach file" ollama-buddy-attach-file)
-   ("w" "Show attachments" ollama-buddy-show-attachments)
+   ("j" "Attach file" ollama-buddy-attach-file)
+   ("l" "Show attachments" ollama-buddy-show-attachments)
    ("d" "Detach file" ollama-buddy-detach-file)
    ("0" "Clear all attachments" ollama-buddy-clear-attachments)
    ("q" "Quit" transient-quit-one)])
@@ -333,9 +358,9 @@
    (lambda () (concat "Web Search - " (ollama-buddy--web-search-status)))
    ["Search"
     ("s" "Search & Display" ollama-buddy-web-search)
-    ("a" "Search & Attach" ollama-buddy-web-search-attach)]
+    ("j" "Search & Attach" ollama-buddy-web-search-attach)]
    ["Manage"
-    ("w" "Show Attachments" ollama-buddy-show-attachments)
+    ("l" "Show Attachments" ollama-buddy-show-attachments)
     ("0" "Clear All" ollama-buddy-clear-attachments)]
    ["Info"
     :description
@@ -372,7 +397,7 @@
   "Transient menu for user system prompts."
   ["User System Prompts"
    [("S" "Save Current" ollama-buddy-user-prompts-save)
-    ("L" "Set as System Prompt" ollama-buddy-user-prompts-load)
+    ("j" "Set as System Prompt" ollama-buddy-user-prompts-load)
     ("N" "Create New" ollama-buddy-user-prompts-create-new)
     ("l" "List All Prompts" ollama-buddy-user-prompts-list)]
    [("e" "Edit Prompt" ollama-buddy-user-prompts-edit)
@@ -428,7 +453,10 @@ Returns the interned command symbol."
           (let* ((group-name (or (plist-get plist :group) "Commands"))
                  (desc (or (plist-get plist :description) (symbol-name name)))
                  (model (plist-get plist :model))
-                 (full-desc (if model (format "%s [%s]" desc model) desc))
+                 (in-buf (eq (plist-get plist :destination) 'in-buffer))
+                 (full-desc (concat desc
+                                    (if model (format " [%s]" model) "")
+                                    (if in-buf " ✎" "")))
                  (sym (ollama-buddy--role-menu-ensure-command cmd-def))
                  (spec (list key-str full-desc sym)))
             (unless (gethash group-name groups)
@@ -441,28 +469,35 @@ Returns the interned command symbol."
             (nreverse group-order))))
 
 (defun ollama-buddy--selection-status ()
-  "Return a string describing the current selection status."
-  (if (use-region-p)
-      (let* ((beg (region-beginning))
-             (end (region-end))
-             (chars (- end beg))
-             (lines (count-lines beg end)))
-        (format "Selection: %d chars, %d lines" chars lines))
-    "No selection"))
+  "Return a string describing the current selection and active mode indicators."
+  (let ((selection (if (use-region-p)
+                       (let* ((beg (region-beginning))
+                              (end (region-end))
+                              (chars (- end beg))
+                              (lines (count-lines beg end)))
+                         (format "Selection: %d chars, %d lines" chars lines))
+                     "No selection"))
+        (in-buffer (if (bound-and-true-p ollama-buddy-in-buffer-replace)
+                       " ✎ In-Buffer" "")))
+    (concat selection in-buffer)))
 
 (defun ollama-buddy-role-transient-menu ()
   "Dynamic role-specific command menu.
 Rebuilds the transient prefix each invocation to reflect the
 current role's `ollama-buddy-command-definitions'."
   (interactive)
-  (let ((group-vectors (ollama-buddy--role-menu-build-groups))
-        (selection-status (ollama-buddy--selection-status)))
+  (let* ((group-vectors (ollama-buddy--role-menu-build-groups))
+         (selection-status (ollama-buddy--selection-status))
+         (toggle-label (if (bound-and-true-p ollama-buddy-in-buffer-replace)
+                           "In-Buffer Replace [ON]  ✎"
+                         "In-Buffer Replace [OFF]")))
     (eval
      `(transient-define-prefix ollama-buddy--role-transient-menu-impl ()
         "Dynamic role-specific command menu."
         [:description
          (lambda () ,selection-status)
-         ,@group-vectors]))
+         ,@group-vectors]
+        [("W" ,toggle-label ollama-buddy-toggle-in-buffer-replace)]))
     (transient-setup 'ollama-buddy--role-transient-menu-impl)))
 
 (provide 'ollama-buddy-transient)
